Ext.define('App.storage.QueryConstructor', {
    singleton: true,
    requires: [
        'App.utility.Date'
    ],

    dateUtility: false,

    constructor: function() {
        this.dateUtility = App.utility.Date;
    },

    /*
    * Exapmle object {
    *     filters: {
    *         bill,
    *         cost,
    *         destination,
    *         duration,
    *         period,
    *         trunks,
    *         tags,
    *         types
    *     },
    *     grouping: ['DATE_DAY', 'SERVICE', 'PBX'] or empty array,
    *     sort: {
    *         field: 'FIELD_NAME',
    *         order: 'ASC / DESC'
    *     },
    *     columns: [
    *       {
    *           name: '',
    *           as: ''
    *       }
    *     ],
    *     limit: {
    *         limit: num,
    *         offset: num
    *     }
    * }
    */
    build: function(queryObject) {
        var queryBuilder = new this.QueryBuilder(this);

        var filters = queryObject['filters'];
        for (var key in filters) {
            if (this.decorators.hasOwnProperty(key)) {
                queryBuilder.decorate(key, filters[key]);
            }
        }

        var columns = queryObject['columns'];
        for (var key in columns) {
            if (this.decorators.hasOwnProperty(columns[key].name)) {
                queryBuilder.decorate(columns[key].name, columns[key]);
            }
        }

        var grouping = queryObject['grouping'];
        for (var key in grouping) {
            if (this.decorators.hasOwnProperty(grouping[key].name)) {
                queryBuilder.decorate(grouping[key].name, grouping[key]);
            }
        }

        if (queryObject['limit'] !== undefined) {
            queryBuilder.decorate('limits', queryObject['limit']);
        }

        if (queryObject['sort'] !== undefined) {
            queryBuilder.decorate('sort', queryObject['sort']);
        }

        return queryBuilder.make(queryObject['getTotalRows'], queryObject['getSumRows']);
    },

    /*Query builder. Pattern: Decorator*/
    QueryBuilder: function(scope) {
        var decoratorsList = [];
        var parent = scope;
        var lazyDecorators = [];

        var queryData = {
            where: [],
            fields: [],
            unionFields: [],
            from: ['`EVENTS` as events_main'],
            joins: [],
            order: '',
            grouping: [],
            limit: '',
            assocForSuperSelect: []
        }

        var decorate = function(decoratorName, data) {
            decoratorsList.push({
                name: decoratorName,
                data: data
            });
        }

        var make = function(getTotalRows, getSumRows) {
            for (var i in decoratorsList) {
                if (decoratorsList[i].name === 'GROUP_BY_USER' || decoratorsList[i].name === 'GROUP_BY_CLIENT' || decoratorsList[i].name === 'GROUP_BY_NUMBER' || decoratorsList[i].name === 'GROUP_BY_GROUP'
                    || decoratorsList[i].name === 'GROUP_BY_COMMUNICATION_OF_USERS' || decoratorsList[i].name === 'GROUP_BY_COMMUNICATION_OF_GROUPS') {

                    lazyDecorators.push(decoratorsList[i]);
                    continue;
                }
                queryData = parent.decorators[decoratorsList[i].name].call(parent, decoratorsList[i].data, queryData);
            }

            var resultQuery = '';

            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT " + queryData.fields.join(',');
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (queryData.grouping.length > 0) {
                resultQuery += " GROUP BY " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += " ) ";
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }
            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            for (var i in lazyDecorators) {
                resultQuery = parent.decorators[lazyDecorators[i].name].call(parent, lazyDecorators[i].data, queryData, getTotalRows, getSumRows);
            }
            return resultQuery;
        }

        return {
            decorate: decorate,
            make: make
        };
    },

    /*Decorators*/
    decorators: {
        /*Filter decorators */
        bill: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.bill = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.bill > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.bill < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        cost: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.cost = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.cost > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.cost < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        duration: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.duration = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.duration > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.duration < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        destination: function(data, queryData) {
            var query = [];

            var subFilters = [
                {
                    keyCheck: 'extToIntEnabled',
                    from: 'extToIntFrom',
                    to: 'extToIntTo',
                    direction: 2
                },
                {
                    keyCheck: 'intToExtEnabled',
                    from: 'intToExtFrom',
                    to: 'intToExtTo',
                    direction: 3
                },
                {
                    keyCheck: 'intToIntEnabled',
                    from: 'intToIntFrom',
                    to: 'intToIntTo',
                    direction: 4
                }
            ];

            for (var i in subFilters) {
                if (!data.hasOwnProperty(subFilters[i].keyCheck) || data[subFilters[i].keyCheck] !== true) {
                    continue;
                }
                var rowQuery = [];
                rowQuery.push(
                    " events_main.direction = " + subFilters[i].direction + " "
                );

                if (data.hasOwnProperty(subFilters[i].from) && data[subFilters[i].from].length > 0) {
                    var elementsQueryFrom = [];
                    for (var j in data[subFilters[i].from]) {
                        //Generate from query element
                        switch (parseInt(data[subFilters[i].from][j].type, 10)) {
                            case 1:
                                //custom phone number
                                var customNumber = data[subFilters[i].from][j].name;
                                elementsQueryFrom.push(
                                    " events_main.`from` " +
                                    ( customNumber && customNumber.indexOf('%')>-1? "like" : "=" ) +
                                    " '" + customNumber + "' "
                                );
                                break;
                            case 2:
                                //call class
                                elementsQueryFrom.push(
                                    " events_main.`class` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 3:
                                //group
                                elementsQueryFrom.push(
                                    " events_main.`from_group` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 4:
                                //local user
                                //TODO: optimization: add type of user
                                elementsQueryFrom.push(
                                    " events_main.`from_user` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 5:
                                //phonebook user
                                //TODO: optimization: add type of user
                                elementsQueryFrom.push(
                                    " events_main.`from_user` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 6:
                                //phonebook company
                                elementsQueryFrom.push(
                                    " events_main.`from_group` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                        }
                    }
                    //Push to array subqueries
                    if (elementsQueryFrom.length > 0) {
                        rowQuery.push(
                            " ( " + elementsQueryFrom.join(" OR ") + " ) "
                        );
                    }
                }

                if (data.hasOwnProperty(subFilters[i].to) && data[subFilters[i].to].length > 0) {
                    var elementsQueryTo = [];
                    for (var j in data[subFilters[i].to]) {
                        //Generate to query element
                        switch (parseInt(data[subFilters[i].to][j].type, 10)) {
                            case 1:
                                //custom phone number
                                var customToNumber = data[subFilters[i].to][j].name;
                                elementsQueryTo.push(
                                    " events_main.`to` " +
                                    ( customToNumber && customToNumber.indexOf('%')>-1? "like" : "=" ) +
                                    " '" + customToNumber + "' "
                                );
                                break;
                            case 2:
                                //call class
                                elementsQueryTo.push(
                                    " events_main.`class` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 3:
                                //group
                                elementsQueryTo.push(
                                    " events_main.`to_group` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 4:
                                //local user
                                //TODO: optimization: add type of user
                                elementsQueryTo.push(
                                    " events_main.`to_user` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 5:
                                //phonebook user
                                //TODO: optimization: add type of user
                                elementsQueryTo.push(
                                    " events_main.`to_user` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 6:
                                // phonebook company
                                elementsQueryTo.push(
                                    " events_main.`to_group` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                        }
                    }
                    //Push to array subqueries
                    if (elementsQueryTo.length > 0) {
                        rowQuery.push(
                            " ( " + elementsQueryTo.join(" OR ") + " ) "
                        );
                    }
                }
                query.push(
                    " ( " + rowQuery.join(" AND ") + " ) "
                );
            }

            if (query.length > 0) {
                query = "(" + query.join(' OR ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        period: function(data, queryData) {
            var query = [];

            switch (parseInt(data.type, 10)) {
                case App.utility.Date.TODAY:
                    //today
                    var today = this.dateUtility.getToday();
                    query.push(" events_main.`date` >= '" + today + "' ");
                    break;
                case App.utility.Date.YESTERDAY:
                    //yesterday
                    today = this.dateUtility.getToday();
                    var yesterday = this.dateUtility.getYesterday();
                    query.push(" events_main.`date` >= '" + yesterday + "' AND events_main.`date` <= '" + today + "' ");
                    break;
                case App.utility.Date.WEEK:
                    //This week
                    var thisWeek = this.dateUtility.getThisWeek();
                    query.push(" events_main.`date` >= '" + thisWeek + "' ");
                    break;
                case App.utility.Date.LAST_2_DAYS:
                    //Last 2 days
                    var last2Days = this.dateUtility.getLastNDays(2);
                    query.push(" events_main.`date` >= '" + last2Days + "' ");
                    break;
                case App.utility.Date.LAST_7_DAYS:
                    //Last 7 days
                    var last7Days = this.dateUtility.getLastNDays(7);
                    query.push(" events_main.`date` >= '" + last7Days + "' ");
                    break;
                case App.utility.Date.PREV_WEEK:
                    //Prev week
                    var prevWeek = this.dateUtility.getPrevWeek();
                    thisWeek = this.dateUtility.getThisWeek();
                    query.push(" events_main.`date` >= '" + prevWeek + "' AND events_main.`date` < '" + thisWeek + " 00:00:00'");
                    break;
                case App.utility.Date.LAST_2_WEEKS:              // LAST_2_WEEKS
                    //Last 2 weeks
                    var last2Weeks = this.dateUtility.getPrevWeek();
                    today = this.dateUtility.getToday();
                    query.push(" events_main.`date` >= '" + last2Weeks + "' AND events_main.`date` <= '" + today + " 23:59:59'");
                    break;
                case App.utility.Date.MONTH:
                    //This month
                    var thisMonth = this.dateUtility.getThisMonth();
                    query.push(" events_main.`date` >= '" + thisMonth + "' ");
                    break;
                case App.utility.Date.LAST_30_DAYS:
                    //Last 30 days
                    var last30Days = this.dateUtility.getLastNDays(30);
                    query.push(" events_main.`date` >= '" + last30Days + "' ");
                    break;
                case App.utility.Date.PREV_MONTH:
                    //Prev month
                    thisMonth = this.dateUtility.getThisMonth();
                    var prevMonth = this.dateUtility.getPrevNMonth(1);
                    query.push(" events_main.`date` >= '" + prevMonth + "' AND events_main.`date` < '" + thisMonth + " 00:00:00'");
                    break;
                case App.utility.Date.LAST_60_DAYS:
                    //Last 60 days
                    var last60Days = this.dateUtility.getLastNDays(60);
                    query.push(" events_main.`date` >= '" + last60Days + "' ");
                    break;
                case App.utility.Date.LAST_3_MONTH:
                    //Last 3 months
                    var last3Months = this.dateUtility.getPrevNMonth(3);
                    query.push(" events_main.`date` >= '" + last3Months + "' ");
                    break;
                case App.utility.Date.LAST_6_MONTH:      // LAST_6_MONTH
                    //Last 6 months
                    var last6Months = this.dateUtility.getPrevNMonth(6);
                    query.push(" events_main.`date` >= '" + last6Months + "' ");
                    break;
                case App.utility.Date.LAST_12_MONTH:
                    //Last 12 months
                    var last12Months = this.dateUtility.getPrevNMonth(12);
                    query.push(" events_main.`date` >= '" + last12Months + "' ");
                    break;
                case App.utility.Date.YEAR:
                    //This year
                    var thisYear = this.dateUtility.getThisYear();
                    query.push(" events_main.`date` >= '" + thisYear + "' ");
                    break;
                case App.utility.Date.PREV_YEAR:
                    //Prev year
                    thisYear = this.dateUtility.getThisYear();
                    var prevYear = this.dateUtility.getPrevNYears(1);
                    query.push(" events_main.`date` >= '" + prevYear + "' AND events_main.`date` <= '" + thisYear + " 23:59:59' ");
                    break;
                case App.utility.Date.CUSTOM:
                    //Custom
                    query.push(" events_main.`date` >= '" + data['fromDate'] + " " + data['fromTime'] + "' AND events_main.`date` <= '" + data['toDate'] + " " + data['toTime'] + "'  ");
                    break;
            }

            queryData.where.push("(" + query.join(' AND ') + ")");

            return queryData;
        },

        trunks: function(data, queryData) {
            if (!data.hasOwnProperty('trunks') || data.trunks.length === 0) {
                return queryData;
            }
            var query = [];
            for (var i in data.trunks) {
                query.push(
                    " events_main.trunk = " + data.trunks[i].id +  " "
                );
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        pbxes: function(data, queryData) {
            if (!data.hasOwnProperty('pbxes') || data.pbxes.length === 0) {
                return queryData;
            }
            var query = [];
            for (var i in data.pbxes) {
                query.push(
                    " events_main.pbx = " + data.pbxes[i].id +  " "
                );
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        tags: function(data, queryData) {
            if (!data.hasOwnProperty('tags') || data.tags.length === 0) {
                return queryData;
            }

            // var tagsEventsTable = 'INNER JOIN `EVENTS_TAGS` as events_tags_join ON events_tags_join.event_id = events_main.id';
            // var tagsTable = 'LEFT JOIN `TAGS` as tags_join ON tags_join.id = events_tags_join.tag_id';

//            if (Ext.Array.indexOf(queryData.joins, tagsEventsTable) === -1) {
//                queryData.joins.push(tagsEventsTable);
//            }
//            if (Ext.Array.indexOf(queryData.joins, tagsTable) === -1) {
//                queryData.joins.push(tagsTable);
//            }

            var names = [];
            for (var i in data.tags) {
                names.push(data.tags[i].name);
            }
            queryData.where.push("(SELECT `tag_id` FROM `EVENTS_TAGS` WHERE `EVENTS_TAGS`.`event_id` = events_main.id AND `EVENTS_TAGS`.`tag_id` IN (SELECT `ID` FROM `TAGS` WHERE `NAME` IN ('"+ names.join("', '") +"')) LIMIT 1) IS NOT NULL");
            // queryData.where.push(" ( " + ids.join(", ") + " ) ");
            return queryData;
        },

        types: function(data, queryData) {
            if (data.call === false && data.fax === false && data.voicemail === false) {
                return queryData;
            }
            var query = [];
            if (data.call === true) {
                query.push(" events_main.type = 1 ")
            }
            if (data.fax === true) {
                query.push(" events_main.type = 2 ")
            }
            if (data.voicemail === true) {
                query.push(" events_main.type = 3 ")
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        miss_call_sec_period: function(data, queryData) {
            if (parseInt( data )>0) {
                queryData.where.push(" ( events_main.bill != 0 or events_main.ring > "+ parseInt( data ) +" ) ");
            }
            return queryData;
        },

        /*Columns decorators*/
        COLUMN_ID: function(data, queryData) {
            queryData.fields.push(
                " events_main.`id` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_RING: function(data, queryData) {
            queryData.fields.push(
                " (events_main.`ring`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_DATE: function(data, queryData) {
            queryData.fields.push(
                " events_main.`date` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DURATION: function(data, queryData) {
            queryData.fields.push(
                " events_main.`duration` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM: function(data, queryData) {
            var tableName = 'LEFT JOIN `USERS` as usr ON usr.id = events_main.from_user';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " events_main.`from` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM_USER: function(data, queryData) {
            var tableName = 'LEFT JOIN `USERS` as usr ON usr.id = events_main.from_user';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " usr.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM_NAME: function(data, queryData) {

            queryData.fields.push(
                " events_main.`from_name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO: function(data, queryData) {

            queryData.fields.push(
                " events_main.`to` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO_USER: function(data, queryData) {
            var tableName = 'LEFT JOIN `USERS` as usr2 ON usr2.id = events_main.to_user';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " usr2.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO_NAME: function(data, queryData) {

            queryData.fields.push(
                " events_main.`to_name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COUNTRY: function(data, queryData) {
            var tableName = 'LEFT JOIN `COUNTRIES` as country_join ON country_join.id = events_main.country';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " country_join.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COUNTRY_AREA: function(data, queryData) {
            var tableName = 'LEFT JOIN `COUNTRIES_AREAS` as country_area_join ON country_area_join.id = events_main.country_area';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " country_area_join.`name` as `" + data.as +  "` "
            );

            return queryData;
        },

        COLUMN_CLASS: function(data, queryData) {

            queryData.fields.push(
                " events_main.`class` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DEST_TYPE: function(data, queryData) {

            queryData.fields.push(
                " events_main.`class` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_BILL: function(data, queryData) {

            queryData.fields.push(
                " events_main.`bill` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST: function(data, queryData) {

            queryData.fields.push(
                " events_main.`cost` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " events_main.`cost_call` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " events_main.`cost_fax` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TRUNK: function(data, queryData) {
            var tableName = 'LEFT JOIN `TRUNKS` as trunks_join ON trunks_join.id = events_main.trunk';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " trunks_join.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_PBX: function(data, queryData) {
            var tableName = 'LEFT JOIN `PBXES` as pbxes_join ON pbxes_join.id = events_main.pbx';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " pbxes_join.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TAG: function(data, queryData) {
//            var tagsEventsTable = 'LEFT JOIN `EVENTS_TAGS` as events_tags_join ON events_tags_join.event_id = events_main.id';
//            var tagsTable = 'LEFT JOIN `TAGS` as tags_join ON tags_join.id = events_tags_join.tag_id';
//
//            if (Ext.Array.indexOf(queryData.joins, tagsEventsTable) === -1) {
//                queryData.joins.push(tagsEventsTable);
//            }
//            if (Ext.Array.indexOf(queryData.joins, tagsTable) === -1) {
//                queryData.joins.push(tagsTable);
//            }

            queryData.fields.push(
                " events_main.`tags` as `" + data.as + "` "
            );

//            queryData.fields.push(
//                " (SELECT GROUP_CONCAT(`tg`.`name`) FROM PUBLIC.EVENTS_TAGS as `et` INNER JOIN PUBLIC.TAGS as `tg` ON `et`.`tag_id` = `tg`.`id` WHERE `et`.`event_id` = `events_main`.`id`) `" + data.as + "` "
//            );
//            queryData.fields.push(
//                " GROUP_CONCAT(`tags_join`.`name`) `" + data.as + "` "
//            );
//            queryData.grouping.push(
//                "`events_main`.`id`"
//            );

            return queryData;
        },

        COLUMN_SERVICE: function(data, queryData) {

            queryData.fields.push(
                " events_main.`type` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_USERFIELD: function(data, queryData) {

            queryData.fields.push(
                " events_main.`userfield` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FAXDATA: function(data, queryData) {

            queryData.fields.push(
                " events_main.`faxdata` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM_GROUP: function(data, queryData) {
            var tableName = 'LEFT JOIN `GROUPS` as groups_join ON groups_join.id = events_main.from_group';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " groups_join.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO_GROUP: function(data, queryData) {
            var tableName = 'LEFT JOIN `GROUPS` as groups_join2 ON groups_join2.id = events_main.to_group';
            if (Ext.Array.indexOf(queryData.joins, tableName) === -1) {
                queryData.joins.push(tableName);
            }

            queryData.fields.push(
                " groups_join2.`name` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_HOUR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`hour` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_HOUR_OF_DAY: function(data, queryData) {

            queryData.fields.push(
                " events_main.`hour_of_day` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DAY: function(data, queryData) {

            queryData.fields.push(
                " events_main.`day` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DAY_OF_WEEK: function(data, queryData) {

            queryData.fields.push(
                " events_main.`day_of_week` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_WEEK: function(data, queryData) {

            // Attention! in sqlite after dividing the integer field we get INTEGER result!
            // Example select 9 / 6; result 1
            queryData.fields.push(
                " (events_main.`day`-4) / 7 * 7 + 4 as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_MONTH: function(data, queryData) {

            queryData.fields.push(
                " events_main.`month` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_MONTH_OF_YEAR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`month_of_year` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_YEAR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`year` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },


        COLUMN_DIRECTION: function(data, queryData) {

            queryData.fields.push(
                " events_main.`direction` as `" + data.as + "` "
            );

            return queryData;
        },

        /*Custom fields*/
        COLUMN_TOTAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " count(events_main.id) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.event_local) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_LOCAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.event_local) * 100 / count(events_main.id) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.event_incoming) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_INCOMING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.event_incoming) * 100 / count(events_main.id) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.event_outgoing) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_OUTGOING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.event_outgoing) * 100 / count(events_main.id) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.event_answered) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_WITHIN_XX_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(case when events_main.ring < "+User.sla_period+" then events_main.event_answered end) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_OVER_XX_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(case when events_main.ring >= "+User.sla_period+" then events_main.event_answered end) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.event_answered) * 100 / count(events_main.id) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_NOT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.event_unanswered) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_NOT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.event_unanswered) * 100 / count(events_main.id) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.duration) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.duration),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.duration_local) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_LOCAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.duration_local),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.duration_incoming) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_INCOMING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.duration_incoming),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.duration_outgoing) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_OUTGOING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.duration_outgoing),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_RING: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.ring) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_RING: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.ring),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },


        COLUMN_TOTAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.bill) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.bill),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.bill_local) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_LOCAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.bill_local),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.bill_incoming) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_INCOMING_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.bill_incoming),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.bill_outgoing) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_OUTGOING_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.bill_outgoing),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.cost) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.cost),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`cost_call`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.`cost_call`),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`cost_fax`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.`cost_fax`),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_CALL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`event_call`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_VOICEMAIL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`event_voicemail`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_FAX: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`event_fax`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },


        /*Limit decorator*/
        limits: function(data, queryData) {
            queryData.limit = " LIMIT " + data.offset + ", " + data.limit + " ";
            return queryData;
        },

        /*Grouping decorators */
        GROUP_BY_CUSTOM_EVENTS_FIELD: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_HOUR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`hour` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_HOUR_OF_DAY: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`hour_of_day` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DAY: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`day` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DAY_OF_WEEK: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`day_of_week` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_WEEK: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " (events_main.`day`-4) / 7 * 7 + 4 as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_MONTH: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`month` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_MONTH_OF_YEAR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`month_of_year` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_YEAR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`year` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_SERVICE: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`type`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DIRECTION: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`direction`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_TRUNK: function(data, queryData) {
            var tableName = 'LEFT JOIN `TRUNKS` as sys_trunks ON sys_trunks.id = events_main.`trunk`';
            queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`trunk`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(sys_trunks.`name`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_PBX: function(data, queryData) {
            var tableName = 'LEFT JOIN `PBXES` as sys_pbxes ON sys_pbxes.id = events_main.`pbx`';
            queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`pbx`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(sys_pbxes.`name`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_TAG: function(data, queryData) {
            queryData.joins.push('LEFT JOIN `EVENTS_TAGS` as sys_events_tags ON sys_events_tags.event_id = events_main.id');
            queryData.joins.push('LEFT JOIN `TAGS` as sys_tags ON sys_tags.id = sys_events_tags.tag_id');

            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                "  ifnull( sys_tags.`id`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull( sys_tags.`name`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_COUNTRY: function(data, queryData) {
            var tableName = 'LEFT JOIN `COUNTRIES` as sys_country ON sys_country.id = events_main.country';
            queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`country`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(sys_country.`name`,'' ) as `" + data.as + "_NAME` "
            );

            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_CLASS: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`class`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_COMPANY: function(data, queryData) {
            var tableName = "INNER JOIN `GROUPS` as sys_groups_company ON sys_groups_company.`type` = 2 AND (sys_groups_company.id = events_main.`from_group` OR sys_groups_company.id = events_main.`to_group`)";
            queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(sys_groups_company.`id`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(sys_groups_company.`name`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            //queryData.where.push("sys_usr.`type` = " + userType);

            return queryData;
        },

        GROUP_BY_USER: function(data, queryData, getTotalRows, getSumRows, userType) {
            if (userType == undefined) {
                userType = 1;
            }

            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
                queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");
            }


            queryData.where.push("sys_usr.`type` = " + userType);

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            resultQuery += getSumRows !== true ? "SELECT events_main.`from_user` as `" + data.as + "`, sys_usr.`name` as `" + data.as + "_NAME` " : "SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? ',' : '';
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `USERS` as sys_usr ON sys_usr.id = events_main.`from_user` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ";

            resultQuery += getSumRows !== true ? "SELECT events_main.`to_user` as `" + data.as + "`, sys_usr.`name` as `" + data.as + "_NAME` " : "SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? ',' : '';
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `USERS` as sys_usr ON sys_usr.id = events_main.`to_user` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) ";

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
                if (queryData.grouping.length > 0) {
                    resultQuery += " , " + queryData.grouping.join(',') + " ";
                }
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        GROUP_BY_CLIENT: function(data, queryData, getTotalRows, getSumRows) {
            return this.decorators.GROUP_BY_USER(data, queryData, getTotalRows, getSumRows, 2);
        },

        GROUP_BY_GROUP: function(data, queryData, getTotalRows, getSumRows) {

            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
                queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            resultQuery += getSumRows !== true ? " SELECT events_main.`from_group` as `" + data.as + "`, sys_groups.`name` as `" + data.as + "_NAME` " : " SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `GROUPS` as sys_groups ON sys_groups.id = events_main.`from_group` AND sys_groups.`type` = 1 ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ";

            resultQuery += getSumRows !== true ? " SELECT events_main.`to_group` as `" + data.as + "`, sys_groups.`name` as `" + data.as + "_NAME` " : "SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `GROUPS` as sys_groups ON sys_groups.id = events_main.`to_group` AND sys_groups.`type` = 1 ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) ";

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }

            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },
        GROUP_BY_NUMBER: function(data, queryData, getTotalRows, getSumRows) {
            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            resultQuery += getSumRows !== true ? " SELECT events_main.`from` as `" + data.as + "` " : "SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ";

            resultQuery += getSumRows !== true ? " SELECT events_main.`to` as `" + data.as + "` " : "SELECT ";
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) ";

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;

        },

        GROUP_BY_COMMUNICATION_OF_USERS: function(data, queryData, getTotalRows, getSumRows) {
            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "_NAME`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "_NAME`");
            }
            queryData.where.push("sys_usr1.`type` = 1");
            queryData.where.push("sys_usr2.`type` = 1");


            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`from_user` as `" + data.as['from'] + "`, events_main.`to_user` as `" + data.as['to'] + "`, "
                            + " sys_usr1.name as `" + data.as['from'] + "_NAME`, sys_usr2.name as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += " SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `USERS` as sys_usr1 ON sys_usr1.id = events_main.`from_user` ";
            resultQuery += " INNER JOIN `USERS` as sys_usr2 ON sys_usr2.id = events_main.`to_user` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ";

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`to_user` as `" + data.as['from'] + "`, events_main.`from_user` as `" + data.as['to'] + "`, "
                            + " sys_usr1.name as `" + data.as['from'] + "_NAME`, sys_usr2.name as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += " SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `USERS` as sys_usr1 ON sys_usr1.id = events_main.`to_user` ";
            resultQuery += " INNER JOIN `USERS` as sys_usr2 ON sys_usr2.id = events_main.`from_user` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) ";

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }

            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        GROUP_BY_COMMUNICATION_OF_GROUPS: function(data, queryData, getTotalRows, getSumRows) {
            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "_NAME`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "_NAME`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`from_group` as `" + data.as['from'] + "`, events_main.`to_group` as `" + data.as['to'] + "`, ";
                resultQuery += " sys_grp1.name as `" + data.as['from'] + "_NAME`, sys_grp2.name as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += "SELECT ";
            }

            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `GROUPS` as sys_grp1 ON sys_grp1.id = events_main.`from_group` ";
            resultQuery += " INNER JOIN `GROUPS` as sys_grp2 ON sys_grp2.id = events_main.`to_group` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ";

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`to_group` as `" + data.as['from'] + "`, events_main.`from_group` as `" + data.as['to'] + "`, ";
                resultQuery += " sys_grp1.name as `" + data.as['from'] + "_NAME`, sys_grp2.name as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += "SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " INNER JOIN `GROUPS` as sys_grp1 ON sys_grp1.id = events_main.`to_group` ";
            resultQuery += " INNER JOIN `GROUPS` as sys_grp2 ON sys_grp2.id = events_main.`from_group` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) ";

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['to'] + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        /*Sort decorators */
        sort: function(data, queryData) {
            if (Ext.isArray(data)) {
                var orders = new Array();

                for (var i=0; i < data.length; i++) {
                    if (data[i]['field'] === 'ID') {
                        orders.push("events_main.`ID` " + data[i]['dest'] + " ");
                    } else {
                        orders.push("`" + data[i]['field'] + "` " + data[i]['dest'] +" ");
                    }
                }

                queryData.order = " ORDER BY " + orders.join(', ') +" ";
            } else if (Ext.isObject(data)) {
                queryData.order = " ORDER BY `" + data['field'] + "` " + data['dest'] +" ";
            }

            return queryData;
        }

    }





});
